_CL Components_ lets you declare Drupal components that you can import and
render in your Drupal site. Other solutions, like
the [Components module](https://www.drupal.org/project/components), only concern
themselves with templates, and not components. But a **template is only a part
of a component**
. [![A component's structure](/files/ksnip_20220313-095710.png)](/files/ksnip_20220313-095710.png)

How to create a component
-------------------------

A component is any directory in your Drupal install that contains
a `metadata.json`. This folder must also contain at least a `my-component.twig`
template. It is advised to create a `css` and `js` directory for your
stylesheets and JS scripts. Learn more about creating a component
in [the documentation](https://git.drupalcode.org/project/cl_components/-/blob/1.x/docs/writing-components.md)
. You will need to let Drupal know where to start scanning for components (
including subdirectories). You can add as many locations as you need in the
settings
page. ![Component settings](/files/0372497880814de187b6bc95707cb402.png)

How to render a component
-------------------------

In Drupal there are two main ways to render some HTML, via a render array, and
via Twig.

### Using a render array

There is a special render element that allows you to render a component based
solely on the component ID and the component props. See this example on
rendering the `my-button` component with the label _Click Me_ using
the `hook_page_top`: 'cl\_component', '#component' => 'my-button', '#variant'
=> 'primary', '#context' => \[ 'text' => t('Click Me'), \], \]; ?> This will
take care of CSS, JS, HTML, and assets for you.

### Via Twig templates

`{% include '@cl_components_example/components/my-button/my-button.twig' with { text: 'Click Me' } %}`

If you want to use the newer `inject` custom syntax, enable `cl_inject`:
`{% inject 'my-card' variant 'warning' with { heading: 'Warning' } %}
  This is the card body.
  {% inject 'my-button' variant 'primary' with { text: 'Click' } %}{% endinject 'my-button' %}
{% endinject 'my-card' %}`

Build a component library!
--------------------------

Now that we have components it is time to build a component library. Thanks
to [CL Server](https://www.drupal.org/project/cl_server) this is super easy! I
recommend [Storybook](https://storybook.js.org), but others are great options as
well. [![Storybook integration](/files/ksnip_20220316-164333.png)](/files/ksnip_20220316-164333.png)
